﻿Imports Microsoft.SharePoint
Imports Microsoft.SharePoint.WebControls
Imports Microsoft.WindowsAzure.StorageClient
Imports Microsoft.WindowsAzure
Imports System.IO
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Text
Imports System.Configuration
Imports System.Web.UI.WebControls


Namespace Layouts.MSSAzureArchive

    Partial Public Class AzureStorage
        Inherits LayoutsPageBase
        Private cloudBlobClient As CloudBlobClient

        Private webSite As SPWeb
        Private webSiteCollection As SPSite
        Private filePath As String
        Private fileToArchive As SPFile

        Private accountName As String
        Private accountSharedKey As String
        Private defaultContainerName As String

        Private cloudBlob As CloudBlob
        Private blobContainer As CloudBlobContainer

        Private blobUri As Uri
        Private queueUri As Uri
        Private tableUri As Uri
        Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
            Me.InitializeCloudStorage()
            If Not IsPostBack Then
                Me.GetContainers()
            End If
        End Sub

        Protected Sub SaveContainer_Click(ByVal o As Object, ByVal e As EventArgs)
            Try
                Dim newContainer = cloudBlobClient.GetContainerReference(Me.newContainerName.Text)
                newContainer.CreateIfNotExist()
                Dim perms = newContainer.GetPermissions()
                perms.PublicAccess = BlobContainerPublicAccessType.Container
                newContainer.SetPermissions(perms)
            Catch ex As Exception
                'display the error on the screen.
                errMessage.Text = ex.Message
            End Try

            GetContainers()
            Me.azureContainers.Items.FindByValue(newContainerName.Text).Selected = True
            Me.azureContainers.Visible = True
            Me.newContainerName.Visible = False
            Me.saveContainer.Visible = False
        End Sub

        Protected Sub CreateContainer_Click(ByVal o As Object, ByVal e As EventArgs)
            Me.azureContainers.Visible = False
            Me.newContainerName.Visible = True
            Me.saveContainer.Visible = True

        End Sub

        Protected Sub Archive_Click(ByVal o As Object, ByVal e As EventArgs)
            Try
                webSite = SPContext.Current.Web
                filePath = webSite.Url.ToString() & Request.QueryString("ItemUrl").ToString()
                fileToArchive = webSite.GetFile(filePath)
                Dim sArchived = fileToArchive.Item("IsArchived").ToString()

                Dim isArchived = Convert.ToBoolean(sArchived)

                If isArchived Then 'Item is flagged as archived
                    errMessage.Text = "This document has already been archived!"
                Else 'Archive to Azure
                    Dim newGuid = Guid.NewGuid().ToString()
                    Dim uniqueBlobName = String.Format(newGuid & "_" & fileToArchive.Name.ToString())

                    blobContainer = cloudBlobClient.GetContainerReference(Me.azureContainers.SelectedValue)
                    blobContainer.CreateIfNotExist()
                    cloudBlob = blobContainer.GetBlockBlobReference(uniqueBlobName)
                    cloudBlob.UploadByteArray(fileToArchive.OpenBinary())

                    Dim docLib = fileToArchive.DocumentLibrary

                    Dim docProperties As New Hashtable()
                    docProperties("IsArchived") = True
                    Dim docLibRelPath = docLib.RootFolder.ServerRelativeUrl
                    Dim docLibPath = String.Empty


                    webSiteCollection = SPContext.Current.Site
                    docLibPath = webSiteCollection.MakeFullUrl(docLibRelPath)

                    Dim documentPath = docLibPath & "/" & fileToArchive.Name.ToString() & ".aspx"
                    Dim azureURL = cloudBlob.Uri.ToString()
                    Dim azureStub = Me.BuildLinkToItem(azureURL).ToString()

                    Dim newFile = webSite.Files.Add(documentPath, _
                                                    UTF8Encoding.UTF8.GetBytes(azureStub), _
                                                    docProperties, _
                                                    True)

                    Dim item = newFile.Item
                    item("Content Type") = "Link to a Document"
                    Dim itemUrl As New SPFieldUrlValue()
                    itemUrl.Description = fileToArchive.Name
                    itemUrl.Url = azureURL
                    item("URL") = itemUrl
                    item("IsArchived") = True
                    item.Update()
                    fileToArchive.Delete()
                End If

            Catch ex As Exception
                errMessage.Text = "Error: " & ex.Message
            End Try
        End Sub

        Private Sub InitializeCloudStorage()
            Try
                accountName = ConfigurationSettings.AppSettings("AccountName")
                accountSharedKey = ConfigurationSettings.AppSettings("AccountSharedKey")
                blobUri = New Uri(ConfigurationSettings.AppSettings("BlobStorageEndPoint"))
                queueUri = New Uri(ConfigurationSettings.AppSettings("QueueStorageEndPoint"))
                tableUri = New Uri(ConfigurationSettings.AppSettings("TableStorageEndPoint"))
                defaultContainerName = ConfigurationSettings.AppSettings("DefaultContainerName").ToString()

                Me.fileName.Text = Request.QueryString("ItemUrl").ToString()

                Dim azAccount As New CloudStorageAccount( _
                    New StorageCredentialsAccountAndKey(accountName, accountSharedKey), _
                    blobUri, queueUri, tableUri)

                cloudBlobClient = azAccount.CreateCloudBlobClient()
            Catch ex As Exception
                errMessage.Text = ex.Message
            End Try
        End Sub

        Private Function BuildLinkToItem(ByVal azureURL As String) As StringBuilder
            Dim azureLinkItem As New StringBuilder()
            Dim linkItemText = _
                "<%@ Assembly Name='Microsoft.SharePoint, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c' %> " & _
                "<%@ Register TagPrefix='SharePoint' Namespace='Microsoft.SharePoint.WebControls' Assembly='Microsoft.SharePoint' %>" & _
                "<%@ Import Namespace='System.IO' %>" & _
                "<%@ Import Namespace='Microsoft.SharePoint' %>" & _
                "<%@ Import Namespace='Microsoft.SharePoint.Utilities' %>" & _
                "<%@ Import Namespace='Microsoft.SharePoint.WebControls' %>" & _
                "<html xmlns:mso=""urn:schemas-microsoft-com:office:office"" xmlns:msdt=""uuid:C2F41010-65B3-11d1-A29F-00AA00C14882"">" & _
                "<Head> <META Name='progid' Content='SharePoint.Link'>" & _
                "<!--[if gte mso 9]><xml>" & _
                "<mso:CustomDocumentProperties>" & _
                "<mso:URL msdt:dt=""string"">http://bing.com,http://bing.com</mso:URL>" & _
                "<mso:ContentType msdt:dt=""string"">Link to a Document</mso:ContentType>" & _
                "</mso:CustomDocumentProperties>" & _
                "</xml><![endif]-->" & _
                "</head>" & _
                "    <body>" & _
                "        <form id='Form1' runat='server'>" & _
                "            <SharePoint:UrlRedirector id='Redirector1' runat='server' />" & _
                "        </form>" & _
                "    </body>" & _
                "</html>"

            azureLinkItem.Append(linkItemText)
            azureLinkItem.Replace("http://bing.com", azureURL)

            Return azureLinkItem
        End Function

        Private Sub GetContainers()
            Dim blobContainers As IEnumerable(Of CloudBlobContainer) = cloudBlobClient.ListContainers()

            Me.azureContainers.DataSource = blobContainers
            azureContainers.DataTextField = "Name"
            Me.azureContainers.DataBind()
            If azureContainers.Items.Count < 1 Then
                Dim defaultContainer As New ListItem(defaultContainerName)
                defaultContainer.Selected = True
                azureContainers.Items.Add(defaultContainer)
            End If
        End Sub
    End Class
End Namespace